function dat = MakeMudimImages(specdat, nphotons, geom, thicknesses,varargin) 
% function dat = MakeMudimImages(specdat, nphotons, geom, thicknesses,varargin) 
% compute images with noise as a function of number of photons
% 
%  inputs: 
%      specdat: a spectrum struct with fields: 
%           egys (x-ray energies) 
%           specnum (photon number spectrum)
%           mus: basis functions for egys, with columns same order as rows in zcorners2 box specs
%      nphotons: the sum of photons in spectrum for all energies
%      geom: a struct with geometry data; fields
%            geom.nx = nx; number x-direction pixels
%            geom.ny = ny;
%            geom.zcorners = zcorners; corners of boxes (complex)
%            geom.ts_object = ts_object; thicknesses of object, one column for each material
%               box heights of each layer assumed to be one
%            geom.isobj = logical(ts_object);
%      thicknesses: actual thicknesses of boxes of regions: one for each layer, same order as rows in zcorners2 box specs
%      verbose: string (optional = false if not present): if present prints progress info to command line.
%      min4normal: (optional = 100) minimum lambda to use normal approx to Poisson
%      normal: string (optional = false if not present) force use of normal approximation to poisson regardless of minimum lambda
%  outputs:
%      dat: a struct with fields: (for now see code)
% based on: MuDimImagesAvar.m 
% REA 10/Aug/2013 10:28 - 15/Aug/2013 9:11
% COPYRIGHT:  Robert E. Alvarez, 2013. All rights reserved.

nreqargs = 4;
if(nargin<nreqargs), error('%d args required, only have %d',nreqargs,nargin);end

verbose = false;
do_normal = false;
min4normal = 100; 
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'verbose';         verbose = true;
     case 'normal';          do_normal = true;
     case 'min4normal';      min4normal=varargin{i+1};     i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end
   % save input parms in output
specdat.specnum_norm = specdat.specnum/trapz(specdat.egys,specdat.specnum);   
dat.specdat = specdat;
dat.thicknesses = thicknesses;
dat.geometry = geom;
   % geometry parameters
ts_object = bsxfun(@times,geom.ts_object,thicknesses); % multiply columns by thicknesses
   % mus parameters
M3 = specdat.mus;
assert(size(M3,2)==3);
M2 = M3(:,1:2);
   % parameters for graybars in images
graybar_range = [2.25 0]; % from black to white
graywedge_position = 'B'; % position of gray wedge to be added to image
graywedge_height = 0.05; % height of gray wedge to be added to image

   % Make the logIs for each pixel in image and subtract air
transmission = exp(-ts_object*specdat.mus');
   % ns_object is expected counts for each pixel, with normalized spectrum incident on object
ns_object = bsxfun(@times,transmission,specdat.specnum_norm'); % size = (ny*nx,nenergy)
log_ns_zero = log(specdat.specnum_norm(:)'); % log of spectrum incident on object

   % prep to invert the logIs_object data to compute the decomposition images
   % use linear MLE with coeffs from known transmitted spectrum in soft tissue region
   % coeffs do not depend on total number of photons so can pre-compute
   % use first pixel in soft tissue region. recall that these are expected values (no noise)
CLinv = diag(ns_object(find(geom.isobj(:,1),1),:)); % inverse covariance of log data. use to compute MLE coeffs
CL = diag(1./ diag(CLinv)); % used for CRLB
cs_logI_to_tfit3 = inv(M3'*CLinv*M3)*M3'*CLinv;
cs_logI_to_tfit2 = inv(M2'*CLinv*M2)*M2'*CLinv;

   % predefine variables for overall images
nNphotons = numel(nphotons); % one column per nphotons values
nyp = geom.ny + 2; % put a boundary between individual images
nxp = geom.nx + 5;
img = 10*ones(3*nyp,nNphotons*nxp); % nbasis = 3 image with a few pixel boundary between images
img2 = 10*ones(3*nyp,nNphotons*nxp); % nbasis=2 image with a few pixel boundary between images
imgsconv = 10*ones(3*nyp,nNphotons*nxp); % conventional image with a few pixel boundary between images
As2dim_with_noise = []; % 2 dim A vectors with noise--use to display whitened data etc.
imgs_opt_proj = []; % optimal projection 'conventional' images from A-vector data
  % predefine stats variables
vars2 = zeros(nNphotons,2);% variances of each A-vector component in soft tissue region for each sigma for nbasis = 2
vars3 = zeros(nNphotons,3); % same as vars2 but for nbasis = 3
CRLB2 = zeros(2,2,nNphotons); % nbasis = 2 CRLB for each thickness
CRLB3 = zeros(3,3,nNphotons);% nbasis = 3 CRLB for each thickness
for ksigma = 1:nNphotons
    N = nphotons(ksigma);
       % the CRLBs
    CRLB2(:,:,ksigma) = CRboundNoLinearize(CL/N,M2); % inv(M2'*(N*CLinv)*M2); 
    CRLB3(:,:,ksigma) = CRboundNoLinearize(CL/N,M3); % inv(M3'*(N*CLinv)*M3); 
       % generate random counts for each pixel and energy bin
    lambdas = N*ns_object;
    min_lambda = min(lambdas(:));
    if (min_lambda>min4normal) || do_normal
        ns_random = randn(size(lambdas)).*sqrt(lambdas) + lambdas;
    else
        if verbose, warning('min lambda %g using slow poisson random number generator',min_lambda);end
        ns_random = poissrnd(lambdas) + 0.1; %add small number so no zeros
    end
    if verbose && (min(ns_random(:))<min4normal), fprintf(' min ns_random %d\n', min(ns_random(:))); end
           % compute noisy logI data from random photon counts ns_random
    logI_plus_noise = -bsxfun(@minus,log(ns_random),log_ns_zero+log(N)); 
    
            % solve for A-vectors at each pixel for nbasis = 3
    ts_fit_with_noise3 = logI_plus_noise*cs_logI_to_tfit3'; % these are A-vectors for the box for this material
    vars3(ksigma,:) = var(ts_fit_with_noise3(geom.is_soft_only,:)); % row vector of variances for each component in soft tissue region

        % put noisy nbasis 3 image data into summary images
    xstart = nxp*(ksigma-1) + 1; % column in overall image to put results
    for kmaterial = 1:3
      img_with_noise = reshape(ts_fit_with_noise3(:,kmaterial)/thicknesses(kmaterial),geom.ny,geom.nx);
      im1 = AddGraywedge2Image(img_with_noise,'range',graybar_range,'fheight',graywedge_height,'position',graywedge_position);
      ystart = nyp*(kmaterial-1)+1;
      img(ystart:(ystart+geom.ny-1),xstart:(xstart+geom.nx-1))= im1;
    end
    
        % put noisy image nbasis 2 data into large summary image
    ts_fit_with_noise2 = logI_plus_noise*cs_logI_to_tfit2'; % these are A-vectors for the box for this material
    vars2(ksigma,:) = var(ts_fit_with_noise2(geom.is_soft_only,:)); % row vector of variances for each component in soft tissue region
    As2dim_with_noise = [As2dim_with_noise ts_fit_with_noise2];
        % put noisy nbasis 2 image data into summary images
    xstart = nxp*(ksigma-1) + 1; % column in overall image to put results
    for kmaterial = 1:2
      img_with_noise = reshape(ts_fit_with_noise2(:,kmaterial)/thicknesses(kmaterial),geom.ny,geom.nx);
      im1 = AddGraywedge2Image(img_with_noise,'range',graybar_range,'fheight',graywedge_height,'position',graywedge_position);
      ystart = nyp*(kmaterial-1)+1;
      img2(ystart:(ystart+geom.ny - 1),xstart:(xstart+geom.nx-1))= im1;
    end
        % photon count only image
    kimg = 1;
    ystart = nyp*(kimg-1)+1;
    imgsconv(ystart:(ystart+geom.ny-1),xstart:(xstart+geom.nx-1)) = ...
        imscale(reshape(-sum(ns_random,2),geom.ny,geom.nx),0); % f (fraction) parameter of imscale is 0 so scales full range to [0,1]

       % optimal projection image in whitened A-vector space
       % see "Near optimal energy selective x-ray imaging system performance with simple detectors"
       %  Med. Phys. 37, 822 (2010); http://dx.doi.org/10.1118/1.3284538
       % first compute whitened A-vectors and optimal pjn angle 
    zts2 = zcomp(ts_fit_with_noise2);
    Cts2 = cov(zcomp(zts2(geom.isobj(:,1))));
    [V,D]=eig(Cts2);
    mwhiten = V*D^-.5; % whiten matrix 
    zswht = zcomp(zcomp(zts2)*mwhiten);
       % now optimal projection image
    angle_opt = angle(mean(zswht(geom.isair)) - ... % optimum projection angle between air ...
        mean(zswht(geom.isobj(:,2))))+pi; % and bone
    kimg = 2;
    ystart = nyp*(kimg-1)+1;
    img_opt = zdot(zswht,exp(1i*angle_opt));
    imgs_opt_proj = [imgs_opt_proj img_opt];
    imgsconv(ystart:(ystart+geom.ny-1),xstart:(xstart+geom.nx-1)) = ...
        imscale(reshape(img_opt,geom.ny,geom.nx),0); % scale result to [0,1]
        % zdot is dot product of 2D vectors represented as complex--gives real output
    if verbose, fprintf('%d ',ksigma); end
end 
dat.Nphotons = nphotons;
dat.img = img;
dat.img2 = img2;
dat.imgsconv = imgsconv;
dat.imgs_opt_proj = imgs_opt_proj;
dat.As2dim_with_noise = As2dim_with_noise;
dat.vars2 = vars2;
dat.CRLB2 = CRLB2;
dat.vars3 = vars3;
dat.CRLB3 = CRLB3;

if verbose 
    fprintf('\n');
end   
