function img = Sim2D(zsiz, varargin)
% function img = Sim2D(zsiz, varargin)
% Compute an image containing geometric shapes specified by variable arguments
% img starts as zeros of specified size and then shapes are added to it
%    inputs:
%      zsiz: a scalar giving the image dimensions, if complex,  nx+1i*ny elseif scalar the side of a square zsiz = zsiz+1i*zsiz
%  a series of optional arguments specifying a variable number of shapes to be added to image.
%  Each shape has a pixel and nx-normalized version depending on how parameters are specified.
%  The shapes are specified as 2D matrices with 1 row per shape 
%  and a variable number of columns depending on the shape as specified below
%        'rect': 3 columns: zcorner1,zcorner2,white_value zcorners are opposite corners (x+iy) pixels
%        'rectn': 3 columns: zcorner1,zcorner2,white_value zcorners are opposite corners (x+iy)/nx 
%        'ellipse': 4 columns: zcenter,zaxes,rotangle,white_value
%        'ellipsen': 4 columns: zcenter/nx,zaxes/nx,rotangle,white_value
%        gaussian: 4 columns: zcenter,zaxes,rotangle,max_value
%        gaussiann: 4 columns: zcenter/nx,zaxes/nx,rotangle,max_value
%    outputs:
%      img: the output image of size (ny,nx) (rows,columns)  
% example:
%      rectspec = [10+10i,90+90i,1;20+30i,60+10i,-.5];
%      img = Sim2D(256,'rect',rectspec);
% REA 5/18-25/09
% ToDo: add shape dependent noise

nreqargs = 1;
assert(nargin>nreqargs);
if isreal(zsiz)
   zsiz = zsiz + 1i*zsiz;
end   
zsiz = round(zsiz);
nx = real(zsiz);
ny = imag(zsiz);
img = zeros(ny,nx);
nxnorm = true;
  % process the optional arguments
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'rect';         spec = varargin{i+1}; assert(size(spec,2)==3); img = img + MxBoxesLoc(zsiz,spec);     i=i+1;
     case 'rectn';         spec = varargin{i+1}; assert(size(spec,2)==3); img = img + MxBoxesLoc(zsiz,spec,nxnorm);     i=i+1;
     case 'ellipse';         spec = varargin{i+1}; assert(size(spec,2)==4); img = img + MxEllipsesLoc(zsiz,spec);     i=i+1;
     case 'ellipsen';         spec = varargin{i+1}; assert(size(spec,2)==4); img = img + MxEllipsesLoc(zsiz,spec,nxnorm);     i=i+1;
     case 'Gaussian';         spec = varargin{i+1}; assert(size(spec,2)==4); img = img + MxGaussiansLoc(zsiz,spec);     i=i+1;
     case 'Gaussiann';         spec = varargin{i+1}; assert(size(spec,2)==4); img = img + MxGaussiansLoc(zsiz,spec,nxnorm);     i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

    % --------------  LOCAL FUNCTIONS --------------------------
    function img = MxGaussiansLoc(zsiz,spec,nx_norm)
    img = zeros(imag(zsiz),real(zsiz));
    nx = real(zsiz);
    for k = 1:size(spec,1)
       zcenter = spec(k,1);
       zaxes = spec(k,2);
        if nargin>2 % spec is nx_norm
           zcenter = nx*zcenter;
           zaxes = nx*zaxes;
        end
        if isreal(zaxes), zaxes = zaxes + 1i*zaxes; end
        sigmas = [real(zaxes) imag(zaxes)];
        rho = sin(pi*spec(k,3)/180);
        sigmas2 = sigmas.^2;
        cv = [sigmas2(1) rho*prod(sigmas);rho*prod(sigmas) sigmas2(2)];
           % use default amplitude
        imgauss = mkGaussian([imag(zsiz),real(zsiz)],cv,[imag(zcenter) real(zcenter)]);
        mx = max(imgauss(:));
        img = img + (spec(k,4)/mx)*imgauss;
    end
    
    function img = MxBoxesLoc(zsiz,spec,nx_norm)
    img = zeros(imag(zsiz),real(zsiz));
    nx = real(zsiz);
    for kbox = 1:size(spec,1)
        if nargin==2 % spec is pixels
           img = img + mkBox(zsiz,spec(kbox,3),spec(kbox,1:2));
        else
           img = img + mkBox(zsiz,spec(kbox,3),nx*spec(kbox,1:2));
        end
    end

    function img = MxEllipsesLoc(zsiz,spec,nx_norm)
    img = zeros(imag(zsiz),real(zsiz));
    nx = real(zsiz);
    for k = 1:size(spec,1)
        if nargin==2 % spec is pixels
           img = img + mkEllipse(zsiz,'zcenter',spec(k,1),'zaxes',spec(k,2),'rotangle',spec(k,3),'white_value',spec(k,4));
        else
           img = img + mkEllipse(zsiz,'zcenter',nx*spec(k,1),'zaxes',nx*spec(k,2),'rotangle',spec(k,3),'white_value',spec(k,4));
        end
    end
